-- Feed An Army: an Elder Scrolls Online add-on
-- by katkat42
-- Adds a button to the Provisioner crafting window that allows you to craft as much of whatever recipe is highlighted as you have ingredients for.
-- This way, you can craft a whole stack of something with one button press, instead of pressing a button, waiting two seconds, pressing a button...
-- Now includes a way to craft more than one but less than all, involving one button press, a slider, and then another button press.

-- initialize some variables
local eventManager = GetEventManager()
local windowManager = GetWindowManager()
local isCooking = false
-- numToMake should always be 0 unless we're in the middle of crafting a stack
local numToMake = 0
local recipeNum
local listNum
multiCraftWindow = nil
-- We add handy buttons to the keybind bar when we're in the provisioning window. 
-- In what language do we label it?
local lang = string.upper(GetCVar("language.2"))
-- if the language is unsupported, default to English
if lang ~= "DE" and lang ~= "FR" then lang = "EN" end
-- first button: craft all
local caButtonLabel = {
	EN = "Craft All",
	DE = "Alles Herstellen",
	FR = "Tout Crafter",
}
local caKeybindDescriptor = {
		name = caButtonLabel[lang],									-- the label for the button
		keybind = "UI_SHORTCUT_TERTIARY",					-- I'm using the generic UI shortcuts so we don't have to have yet another keybinding
		callback = function() FAA_CraftAllStart() end,		-- This is the function that runs when we press the key
		enabled = function(binding, group)					-- This function runs when the keybind strip asks, "Should you be enabled right now?"
			return multiCraftWindow:IsHidden()
		end,
}
-- second button: Craft Many
local cmButtonLabel = {
	EN = "Craft Many",
	DE = "Viele Herstellen",
	FR = "Plusieurs Crafter",
}
local cmKeybindDescriptor = {
	name = cmButtonLabel[lang],
	keybind = "UI_SHORTCUT_PRIMARY",
	callback = function() FAA_ShowFrame() end,
	enabled = function(binding, group)
		return multiCraftWindow:IsHidden()
	end,
}


-- These are the functions that do stuff
-- FAA_CraftSomeStart() is the function that is initially called when you press the "Craft Multiple" button. It displays a box asking how many to craft.
function FAA_ShowFrame()
	-- display a window asking how many to craft
	multiCraftWindow:SetHidden(false)
	KEYBIND_STRIP:UpdateKeybindButtonGroup(PROVISIONER.mainKeybindStripDescriptor)
end

-- FAA_HideFrame() hides the "Craft Multiple" frame.
function FAA_HideFrame()
	multiCraftWindow:SetHidden(true)
	KEYBIND_STRIP:UpdateKeybindButtonGroup(PROVISIONER.mainKeybindStripDescriptor)
end

-- FAA_CraftSome()  is the function that is called when you close the "Craft Multiple" box by clicking "Craft". It crafts the first one of the bunch.
function FAA_CraftSome()
	FAA_HideFrame()
	-- craft the first one here
	-- this should never be false, but just in case...
	if (isCooking) then
		-- can we make this item? if not, do nothing.
		local craftable = PROVISIONER:IsCraftable()
		if (craftable) then
			recipeNum = PROVISIONER:GetSelectedRecipeIndex()
			listNum = PROVISIONER:GetSelectedRecipeListIndex()
			numToMake = multiCraftWindow.slide:GetValue()
			-- we look for 2 spaces in inventory because CraftProvisionerItem() calls CheckInventorySpaceAndWarn(1) even when there 
			-- is already a partial stack of the crafting result in inventory. So to avoid CraftProvisionerItem() from failing the second time it runs,
			-- we require an extra space.
			local slotsNeeded = math.ceil(numToMake/100) + 1
			local haveSpace = CheckInventorySpaceAndWarn(slotsNeeded)
			if (haveSpace) then 
		-- 	craft the first item. When it's done, EVENT_CRAFT_COMPLETED will fire, and the callback FAA_CraftTheStack() will execute.
				CraftProvisionerItem(listNum, recipeNum)
				numToMake = numToMake - 1
			end
		end
	end
end

-- FAA_CraftAllStart() is the function that is initially called when you press the "Craft All" button.
function FAA_CraftAllStart()
	-- this should never be false, but just in case...
	if (isCooking) then
		-- can we make this item? if not, do nothing.
		local craftable = PROVISIONER:IsCraftable()
		if (craftable) then
			recipeNum = PROVISIONER:GetSelectedRecipeIndex()
			listNum = PROVISIONER:GetSelectedRecipeListIndex()
			-- I would love to use a getter function for this like a good girl should, but I couldn't find one. Oh well.
			numToMake = PROVISIONER.recipeTree.selectedNode.data.numCreatable
			-- we look for 2 spaces in inventory because CraftProvisionerItem() calls CheckInventorySpaceAndWarn(1) even when there 
			-- is already a partial stack of the crafting result in inventory. So to avoid CraftProvisionerItem() from failing the second time it runs,
			-- we require an extra space.
			local slotsNeeded = math.ceil(numToMake/100) + 1
			local haveSpace = CheckInventorySpaceAndWarn(slotsNeeded)
			if (haveSpace) then 
		-- 	craft the first item. When it's done, EVENT_CRAFT_COMPLETED will fire, and the callback FAA_CraftTheStack() will execute.
				CraftProvisionerItem(listNum, recipeNum)
				numToMake = numToMake - 1
			end
		end
	end
end

-- FAA_CraftTheStack() is a callback function that runs every time you finish crafting an item in the provisioning window. 
-- That is, it runs when the event EVENT_CRAFT_COMPLETED fires. It checks to see if you need to craft another item, 
-- and if so, starts the process again.
function FAA_CraftTheStack()
	local craftable = PROVISIONER:IsCraftable()
	if (isCooking and numToMake > 0 and craftable) then
		CraftProvisionerItem(listNum, recipeNum)
		numToMake = numToMake - 1
	end
end

-- if we're in the process of crafting an item RIGHT NOW, set a flag that lets us know, so we can hide our keybind on the keybind strip.
function FAA_StartCrafting()
	KEYBIND_STRIP:UpdateKeybindButtonGroup(PROVISIONER.mainKeybindStripDescriptor)
end

-- FAA_OpenCrafting() is a callback function that runs every time you open a crafting window, that is, when EVENT_CRAFTING_STATION_INTERACT	
-- fires. It checks to see if the crafting window you opened is the Provisioning window, and if so, sets up some stuff.
function FAA_OpenCrafting(event, craftType)
	if craftType == CRAFTING_TYPE_PROVISIONING then
		eventManager:RegisterForEvent("FeedAnArmyDoneCraftingListener", EVENT_CRAFT_COMPLETED, function() FAA_CraftTheStack() end)
		eventManager:RegisterForEvent("FeedAnArmyCraftListener",EVENT_CRAFT_STARTED, function() FAA_StartCrafting() end)
		isCooking = true
	else
		-- isCooking "should" already be false, but just in case...
		isCooking = false
	end
end

-- FAA_CloseCrafting() is a callback function that runs every time you close a crafting window, that is, when
-- EVENT_END_CRAFTING_STATION_INTERACT fires. It runs regardless of whether the crafting window was Provisioning
-- or not, because if it wasn't Provisioning, these functions will harmlessly do nothing.
function FAA_CloseCrafting()
	eventManager:UnregisterForEvent("FeedAnArmyDoneCraftingListener")
	eventManager:UnregisterForEvent("FeedAnArmyCraftListener")
	-- just in case we close in the middle of a stack craft
	numToMake = 0
	isCooking = false
end

-- This function creates the "Craft Multiple" window if it isn't created already. It should only run once. 
-- Also, holy crap does it take a lot of lines of code to set up one UI element!
function FAA_CreateMultiWindow()
	-- First, create the window container itself
	multiCraftWindow = windowManager:CreateControl("FeedAnArmyFrame", ZO_ProvisionerTopLevel, CT_CONTROL)
	multiCraftWindow:SetAnchor(BOTTOM, ZO_ProvisionerTopLevelTooltip, TOP, 0, -35)
	multiCraftWindow:SetHidden(true)
	multiCraftWindow:SetMouseEnabled(true)
	multiCraftWindow:SetKeyboardEnabled(true)
	multiCraftWindow:SetMovable(false)
	multiCraftWindow:SetResizeToFitDescendents(true)
	
	-- Add a background
	multiCraftWindow.backdrop = windowManager:CreateControlFromVirtual("FeedAnArmyBackdrop", multiCraftWindow, "ZO_DefaultBackdrop")
	multiCraftWindow.backdrop:SetAnchorFill(multiCraftWindow)
	
	-- Ask the question, "How Many?"
	multiCraftWindow.title = windowManager:CreateControl("FeedAnArmyTitle", multiCraftWindow, CT_LABEL)
	multiCraftWindow.title:SetAnchor(TOP, multiCraftWindow, TOP, 0, 5)
	multiCraftWindow.title:SetFont("EsoUi/Common/Fonts/Univers67.otf|30|soft-shadow-thin")
	multiCraftWindow.title:SetText("How Many?")
	multiCraftWindow.title:SetStyleColor(0,0,0,1)
	
	-- Have a nifty little slider that lets the user input how many
	multiCraftWindow.slide = windowManager:CreateControlFromVirtual("FeedAnArmySlider", multiCraftWindow, "ZO_Slider")
	multiCraftWindow.slide:SetAnchor(TOP, multiCraftWindow.title, BOTTOM, 0, 5)
	multiCraftWindow.slide:SetMouseEnabled(true)

	-- Have a display for the slider's value
	multiCraftWindow.value = windowManager:CreateControl("FeedAnArmyValue", multiCraftWindow, CT_LABEL)
	multiCraftWindow.value:SetAnchor(TOP, multiCraftWindow.slide, BOTTOM, 0, 5)
	multiCraftWindow.value:SetFont("EsoUi/Common/Fonts/Univers67.otf|23|soft-shadow-thin")
	multiCraftWindow.value:SetStyleColor(0,0,0,1)
	
	-- make a container for our buttons
	multiCraftWindow.buttons = windowManager:CreateControl("FeedAnArmyButtons", multiCraftWindow, CT_CONTROL)
	multiCraftWindow.buttons:SetAnchor(TOP, multiCraftWindow.value, BOTTOM, 0, 5)
	multiCraftWindow.buttons:SetResizeToFitDescendents(true)
	multiCraftWindow.buttons:SetKeyboardEnabled(true)
	
	-- have a button when the user is done choosing how many
	multiCraftWindow.buttons.ok = windowManager:CreateControlFromVirtual("FeedAnArmyGo", multiCraftWindow.buttons, "ZO_DialogButton")
	multiCraftWindow.buttons.ok:SetAnchor(LEFT, multiCraftWindow.buttons, LEFT, 0, 0)
	ZO_KeybindButtonTemplate_Setup(multiCraftWindow.buttons.ok, "UI_SHORTCUT_PRIMARY", function() FAA_CraftSome() end, "Craft")
	multiCraftWindow.buttons.ok:SetKeyboardEnabled(true)
	
	-- have a button to cancel the "craft some" operation
	multiCraftWindow.buttons.cancel = windowManager:CreateControlFromVirtual("FeedAnArmyCancel", multiCraftWindow.buttons, "ZO_DialogButton")
	multiCraftWindow.buttons.cancel:SetAnchor(LEFT, multiCraftWindow.buttons.ok, RIGHT, 5, 0)
	ZO_KeybindButtonTemplate_Setup(multiCraftWindow.buttons.cancel, "UI_SHORTCUT_NEGATIVE", function() FAA_HideFrame() end, "Cancel")
	multiCraftWindow.buttons.cancel:SetKeyboardEnabled(true)

	-- Have the display update when the slider's value changes
	multiCraftWindow.slide:SetHandler("OnValueChanged", function(...) multiCraftWindow.value:SetText(tostring(multiCraftWindow.slide:GetValue())) end)

	-- Prepopulate the values for both the slider and display each time the dialog is opened
	multiCraftWindow:SetHandler("OnEffectivelyShown", function(...)
		multiCraftWindow.slide:SetMinMax(1, PROVISIONER.recipeTree.selectedNode.data.numCreatable or 1)
		multiCraftWindow.slide:SetValue(PROVISIONER.recipeTree.selectedNode.data.numCreatable or 1)
		multiCraftWindow.slide:SetValueStep(1)
		multiCraftWindow.value:SetText(tostring(PROVISIONER.recipeTree.selectedNode.data.numCreatable or 1))
	end )
end

-- This runs when the UI is loaded, when EVENT_ADD_ON_LOADED fires.
local function FAA_Initialize(eventType, addonName)
	if addonName ~= "FeedAnArmy" then return end
	-- once we get this far, we only need to do this once, so we can stop listening for the event.
	eventManager:UnregisterForEvent("FeedAnArmyInitialize", EVENT_ADD_ON_LOADED)
	if multiCraftWindow == nil then FAA_CreateMultiWindow() end
	-- add our new button to the provisioner keybind group
	PROVISIONER.mainKeybindStripDescriptor[1].enabled = function(binding, group) return multiCraftWindow:IsHidden() end
	caKeybindDescriptor.visible = PROVISIONER.mainKeybindStripDescriptor[1].visible
	cmKeybindDescriptor.visible = PROVISIONER.mainKeybindStripDescriptor[1].visible
	table.insert(PROVISIONER.mainKeybindStripDescriptor, caKeybindDescriptor)
	table.insert(PROVISIONER.mainKeybindStripDescriptor, cmKeybindDescriptor)
	-- register for some events
	eventManager:RegisterForEvent("FeedAnArmyOpenListener", EVENT_CRAFTING_STATION_INTERACT, function(...) FAA_OpenCrafting(...) end)
	eventManager:RegisterForEvent("FeedAnArmyCloseListener", EVENT_END_CRAFTING_STATION_INTERACT, function(...) FAA_CloseCrafting(...) end)
end

-- ... and this ensures that the above function gets run when it is time.
eventManager:RegisterForEvent("FeedAnArmyInitialize", EVENT_ADD_ON_LOADED, function(...) FAA_Initialize(...) end)